var Slm = require('./slm');

function Interpolation() {
  this.escapedInterpolationRe = /^\\\$\{/;
  this.interpolationRe = /^\$\{/;
  this.staticTextRe = /^([\$\\]|[^\$\\]*)/;
}

var InterpolationProto = Interpolation.prototype = new Slm;

InterpolationProto.on_slm_interpolate = function(exps) {
  var str = exps[2], m, code;

  // Interpolate variables in text (#{variable}).
  // Split the text into multiple dynamic and static parts.
  var block = ['multi'];
  do {
    // Escaped interpolation
    if (m = this.escapedInterpolationRe.exec(str)) {
      block.push(['static', '${']);
      str = str.slice(m[0].length);
    } else if (m = this.interpolationRe.exec(str)) {
      // Interpolation
      var res = this.parseExpression(str.slice(m[0].length));
      str = res[0], code = res[1];
      var escape = code[0] !== '=';
      block.push(['slm', 'output', escape, escape ? code : code.slice(1), ['multi']]);
    } else if (m = this.staticTextRe.exec(str)) {
      // Interpolation
      block.push(['static', m[0]]);
      str = str.slice(m[0].length);
    }
  } while (str.length);

  return block;
}

InterpolationProto.parseExpression = function(str) {
  for (var count = 1, i = 0, l = str.length; i < l && count; i++) {
    if (str[i] === '{') {
      count ++;
    } else if (str[i] === '}') {
      count --;
    }
  }

  if (count) {
    throw new Error('Text interpolation: Expected closing }');
  }

  var code = str.substring(0, i - 1);
  return [str.slice(i), code];
}

module.exports = Interpolation;
